function [sol,ierr,output,history,grad,diagnostic]=CoDoSol(x,F,l,u,tol,parms,Fjac)
%  
%   Globally convergent  solver for constrained nonlinear
%   systems of equations
%
%           F(x)=0         l<=x<=u
%
%   where  F: R^n --> R^n. 
%   The algorithm combines Newton method and a (either spherical or elliptical)
%   trust-region procedure. The merit function used is f(x)=norm(F(x)).
%   The elliptical trust-region is defined employing the scaling 
%   diagonal matrix D used to compute the scaled gradient.
%   The trust-region problem is approximately solved by a "constrained dogleg" method.
%   Strictly feasible iterates are generated.
%   For more details, see
%
%      http://codosol.de.unifi.it
%
%   Uses  dmatrici.m, diffjac.m
%   
%   The problem to be solved has to be provided defining the following function(s):
%
%           - function_name.m
%                function [y] = function_name(x)
%                Input:
%                   x = point in which the function has to be evaluated
%                Output:
%                   y = F(x)
%           - Jac_function_name.m (optional)
%                function [J] = Jac_function_name(x)
%                Input:
%                   x = point in which the Jacobian has to be evaluated
%                Output:
%                   J = F'(x)
%
%
%    Further, the following function may be provided if the user would like to
%    apply his own scaling matrix:
%
%           - usersd.m (optional)
%                function [d] = usersd(x,grad,l,u)
%                Input:
%                   x = point in which the scaling matrix has to be evaluated
%                grad = gradient of f, preevaluated
%                 l,u = constraints
%                Output:
%                   d = array such that d=diag(D(x)).
%
%
%   REMARK: If the user would like to work with SPARSE MATRICES, and
%           function Jac_function_name.m is not provided, it is only required that function
%           function_name.m provides sparse output.
%           Then, the approximated Jacobian will be automatically computed as a
%           sparse matrix. If function Jac_function_name.m is provided by the user, then also
%           its output has obviously to be a sparse matrix. 
%           When the Jacobian is stored as a sparse matrix, the Newton step is
%           computed via the built-in Matlab function LU with the syntax as to use UMFPACK.  
%
%
%
%
%   Typical usages:
%       
%   function [sol,ierr,output]=CoDoSol(x,F,l,u,tol,parms)
%   ---------------------------------------------------
%  
%   The Jacobian matrix of F is approximated using 
%   finite-differences.
%
%   Input:
%  
%     x     = initial iterate x_0.
%     F     = function that accepts input vector x and returns the 
%             vector F(x).
%     l,u   = vectors containing the lower and upper bounds on the
%             variables.
%             set l(i) = -Inf if x(i) is unbounded below; 
%             set u(i) = Inf if x(i) is unbounded above.
%     tol   = [atol,rtol] absolute/relative error tolerances used 
%             in the stopping criterion:
%                    norm(F(x))<=atol+rtol*norm(F(x_0)).
%           =[] the default value [1.0e-6,0] is used.
%     parms = [maxit,maxnf,tr,delta,scaling,outflag]
%             maxit=  maximum number of nonlinear iterations;
%             maxnf=  maximum number of F-evaluations (F-evaluations 
%                     due to the Jacobian approximation are not included);
%                tr= choice of the trust-region type
%                      0  Spherical trust-region (G=I)
%                      1  Elliptical trust-region (G=D^(-1/2))
%             delta=  choice of the initial trust region radius Delta. 
%                     -1  then Delta=1;
%                     -2  then Delta=norm(G(x_0)^(-1)grad(f(x_0)));
%                     >0  then Delta=delta.
%           scaling= choice of the scaling matrix
%                     <0   Kanzow-Klug scaling matrix, with gamma=-scaling
%                      0   Coleman-Li scaling matrix
%                      1   Hager et al. scaling matrix
%                      2   User supplied scaling matrix
%           outflag=  printlevel 
%                     0   The final summary output is printed:
%                            - number of performed iterations,  
%                            - number of performed F-evaluations (F-evaluations due to
%                              the Jacobian approximation are not included);
%                            - norm(F(sol))
%                     >0  one line of summary output for each iteration is printed:
%                            - iteration number k;
%                            - norm(F(x_k));
%                            - number of trust region radius reductions;  
%                            - value of gamma used in the path computation;
%                            - ratio of two successive nonlinear residuals;
%
%           =[] the default value [300 1000 1 -1 0 2] is used.
%
%   Output:
%     sol    =   final estimate of the solution.
%     ierr   = error indicator:
%              0   upon successful termination;
%              1   the limiting number of iterations has been reached; 
%              2   the limiting number of F-evaluations has been reached;
%              3   the trust region radius Delta has become too small 
%                  (Delta<sqrt(eps));
%              4   no improvement for the nonlinear reasidual could be obtained:
%                  abs(norm(F(x_k))-norm(F(x_{k-1})))<=100*eps*norm(F(x_k));
%              5   the sequence has approached a minumum of f in the box:
%                  norm(D(x_k)*grad(f(x_k)))<100*eps. 
%              6   an overflow would be generated when computing the scaling 
%                  matrix D since the sequence is approaching a bound.       
%     output =   vector containing:
%                        - number of performed iterations;
%                        - number of performed F-evaluations (F-evaluations due to 
%                                       the Jacobian approximation are not included); 
%                        - norm(F(sol));
%                        - norm(D(sol)grad(f(sol)));  
%                        - total number of reductions of the trust region radius. 
%
%
%
%
%  function [sol,ierr,output]=CoDoSol(x,F,l,u,tol,parms,Fjac)
%  --------------------------------------------------------
% 
%  Solves as above with the Jacobian matrix of F evaluated analitically 
%  by the user-supplied function Fjac. Function Fjac(X) must  
%  returns the Jacobian matrix of the function F evaluated at x. 
%
%
%
%
%  function [sol,ierr,output,history]=CoDoSol(x,F,l,u,tol,parms,..) 
%  -------------------------------------------------------------- 
%  
%  Also returns a matrix named history that describes the convergence 
%  history of CoDoSol. Each row of history contains:
%                        - iteration number k; 
%                        - norm(F(x_k));
%                        - number of  trust region radius reductions;  
%                        - value of gamma used in the path computation;
%                        - ratio of two successive nonlinear residuals. 
% 
%
%
%
%  function [sol,ierr,output,history,grad]=CoDoSol(x,F,l,u,tol,parms,..)
%  -------------------------------------------------------------------
%
%  Also returns the gradient grad of the merit function f at sol.
%
%
%
%
%  function [sol,ierr,output,history,grad,diagnostic]=CoDoSol(x,F,l,u,tol,parms,..)
%  ------------------------------------------------------------------------------- 
%
%  If full matrices are used, also returns some diagnostic information: 
%                 diagnostic(1)=rank of the Jacobian matrix at sol, 
%                               computed by the Matlab function rank;
%                 diagnostic(2:n+1)=singular values of the Jacobian 
%                                   matrix at sol, computed by the Matlab 
%                                   function svd.
%
%
%
%  function [sol,ierr,output]=CoDoSol(x,F,l,u,tol)
%  function [sol,ierr,output]=CoDoSol(x,F,l,u)
%  --------------------------------------------------------
% 
%  The following default values are used for tol and/or parms (the same applies in general if
%  tol and/or parms are assigned empty values):
%
%             tol:
%                absolute tolerance = 1.0e-6
%                relative tolerance = 0
%           parms:
%                300 as maximum number of nonlinear iterations
%                1000 as maximum number of function evaluations
%                elliptical trust-region
%                Initial trust-region radius Delta_0=1
%                Coleman-Li scaling matrix
%                one line of summary output for each iteration is printed
%
%  Internal parameters:
%
%   t=0.25                : used for accuracy requirements;
%   thetal=0.99995,sigma=0.99995 : used to ensure strictly feasible iterates; 
%   delta1=0.25,delta2=2         : used to update the trust-region size; 
%   w=0.75                       : parameter that governs the increase of the 
%                                  trust-region size.
%


%
%   INITIALIZATION
%

n=length(x);  
ierr=0; nridut=0;
itc=0;
nvf=0; 

if any(x<=l|x>=u)
   disp('ATTENTION: the initial guess is not strictly feasible')
   return
end

fx=feval(F,x);
nvf=nvf+1; fnrm=norm(fx);
fnrm2=fnrm^2;
%
%   DEFAULT VALUES FOR TOL AND/OR PARMS
%
if nargin <=4 | isempty(tol)
   tol = [1.0e-6  0];
end
if nargin <=5 | isempty(parms)
   parms = [300 1000 1 -1 0 2];
end

%
%   PARAMETER SETTINGS
%
epsilon=100*eps;
Deltamin=sqrt(eps);
atol=tol(1); rtol=tol(2); stoptol=atol+rtol*fnrm;
maxit=parms(1); maxnf=parms(2); tr=parms(3); Delta=parms(4); scal=parms(5); outflag=parms(6);

%   
%   INTERNAL PARAMETERS
%
r=0.1; t=0.25; w=0.75; delta1=0.25; delta2=2; thetal=0.99995; sigma=0.99995;
%
%   INITIAL OUTPUT
%
if nargout>=4
     history(1,:)=[itc,fnrm,0,0,0];
end
if outflag>0
    disp(sprintf('\n'))
    disp(sprintf('%s it %s ||F||_2 %s rid_step %s gamma %s ratio',...
                  blanks(3),blanks(4),blanks(2),blanks(2),blanks(7)))
end 
%
%   ITERATION
%
count_bb=0;
lambda=0;
grad=ones(n,1);
while (fnrm>stoptol & itc<maxit & nvf<maxnf)
     itc=itc+1;
     fnrm0=fnrm;
%
%   JACOBIAN EVALUATION
%
     if nargin==7
        jac=feval(Fjac,x);
     else
        jac=diffjac(x,F,fx,l,u);
     end
     grad_old=grad;
     grad= jac'*fx;
%
%   CALCULATION OF THE SCALING MATRICES D (D), D^1/2 (DSQR), AND INV(D^1/2) (DMSQR)
%
     if scal==1
       if itc==1
        lambda=max(1.d-2,norm(grad));
       else
        lambda=max(1.d-2,(p'*(grad-grad_old))/(p'*p));
       end
     end
     
     [d,dsqr,dmsqr,ierr]=dmatrici(x,grad,l,u,scal,lambda); 
     if ierr==6
        break;
     end
  
%
%   TRUST-REGION TYPE ASSIGNMENT
%     
     if tr==0
       G = ones(n,1); 
     else
       G = dmsqr;
     end

     dsqrgrad=dsqr.*grad;
     dgrad=d.*grad;
     Gdgrad=G.*dgrad;
     jdgrad=jac*dgrad;
     Gmgrad=grad./G;

     ndsqrgrad=norm(dsqrgrad);
     ndgrad=norm(dgrad);
     nGdgrad=norm(Gdgrad);
     njdgrad=norm(jdgrad);
     nGmgrad=norm(Gmgrad);

     if ndgrad<epsilon
        ierr=5;
        break;
     end     

%
%   COMPUTATION OF THE MINIMIZER (PC) OF THE QUADRATIC MODEL ALONG D*GRAD 
%
     vert=(ndsqrgrad/njdgrad)^2; 
     pc=-vert*dgrad;
     pcv=pc;
     npc=norm(pc);
     pcc=pc;

%
%   INITIAL TRUST REGION RADIUS
%
     if itc==1
        if Delta==-1
           Delta=1; 
        else
           if Delta==-2
              Delta=nGmgrad;
           end
        end
     end
%
%   NEWTON STEP
%
      if ~issparse(jac) | ~isnew
             [L,U,P] = lu(jac);
      else
             [L,U,P,Q] = lu(jac);
      end

      jacsingular = 0;
      if min(abs(diag(U)))== 0
         jacsingular = 1;
      end

      if ~jacsingular
         if ~issparse(jac) | ~isnew
             sn=-L\(P*fx);
             sn=U\sn;
         else
             sn=-L\(P*fx);
             sn=U\sn;
             sn=Q*sn;
         end

%
%   PROJECTED NEWTON STEP
%
         if (x+sn)>l & (x+sn)<u  
            indpr=0; 
         else
            indpr=1;  
            sn=max(x+sn,l)-x;
            sn=min(x+sn,u)-x;
            sn=max(sigma,1-norm(sn))*sn;
         end
      end
%
%   TRUST-REGION STRATEGY
%
      rhof=0; nridu=-1;
      while (rhof<t & Delta>Deltamin)    
            nridu=nridu+1;           
%
%   COMPUTATION OF THE CAUCHY POINT
%
           if vert*nGdgrad>Delta    %norm(dmsqr.*pcc)>Delta
              pcv=-Delta*dgrad/nGdgrad;
           end
%         
%   COMPUTATION OF THE TRUNCATED CAUCHY POINT
%
           pciv=pcv;
           npcv=norm(pcv);
           for i=1:n
               if pcv(i)~=0
                  alp(i)=max((l(i)-x(i))/pcv(i),(u(i)-x(i))/pcv(i));
               else
                  alp(i)=Inf;
               end
           end        
           alpha1=min(alp);
           if alpha1<=1
              pciv=max(thetal,1-npcv)*alpha1*pcv;
           end
%          
%   PATH COMPUTATION 
%
           if ~jacsingular
%
%   THE JACOBIAN IS NONSINGULAR
%
              aa=fx+jac*pciv;
              seg=sn-pciv;
              bb=jac*seg;
              if norm(bb)~=0
                 gammamin=-aa'*bb/norm(bb)^2;
                 a=norm(G.*seg)^2;
                 b=(G.*pciv)'*(G.*seg);
                 c=norm(G.*pciv)^2-Delta^2;
                 l1=(-b+sign(-b)*sqrt(b^2-a*c))/a;
                 l2=c/(l1*a);
                 if gammamin>0
                    gammaend=max(l1,l2);
                    gamma=min(gammamin,gammaend);
                    if gamma >1
                       for i=1:n
                           if seg(i)~=0 
                              alp(i)=max((l(i)-(x(i)+pciv(i)))/seg(i),...
                                         (u(i)-(x(i)+pciv(i)))/seg(i));
                           else
                              alp(i)=Inf;
                           end
                       end        
                       alpha1=min(alp);
                       gamma=min(thetal*alpha1,gamma); 
                    end
                 else
                    gammaend=min(l1,l2);
                    gamma=max(gammamin,gammaend);
                    if gamma<0
                       for i=1:n
                           if seg(i)~=0 
                              alp(i)=max((l(i)-(x(i)+pciv(i)))/(-seg(i)),...
                                         (u(i)-(x(i)+pciv(i)))/(-seg(i)));
                           else
                              alp(i)=Inf;
                           end
                       end        
                       alpha1=min(alp);
                       gamma=max(-thetal*alpha1,gamma);
                    end
                 end
              else
                 gamma=0;
              end
              p=(1-gamma)*pciv+gamma*sn;
           else
%   THE JACOBIAN IS SINGULAR
              p=pciv;
              gamma=0;
           end
           
%
%   ACCURACY REQUIREMENTS
%
          
           xpp=x+p;
           fxpp=feval(F,xpp);
           nvf=nvf+1;
           fnrmxpp=norm(fxpp);
           rhof=(fnrm-fnrmxpp)/(fnrm-norm(fx+jac*p));
           Deltas=Delta;
           Delta=min(delta1*Delta,0.5*norm(G.*p));  
     end

     if (Delta<=Deltamin & rhof<t)
        nridu=nridu+1;
        ierr=3;
        break
     end 
     Delta=Deltas;
%  
%   UPDATING OF THE ITERATE
%
     x=xpp;
     del=100*eps;
     inddel=0;
     for i=1:n
         if x(i)<l(i)+del  
            x(i)=l(i)+del;
            inddel=1;
         end 
         if x(i)>u(i)-del  
            x(i)=u(i)-del;
            inddel=1;
         end
     end

     if inddel==0
        fx=fxpp;
     else
        fx=feval(F,x);
        nvf=nvf+1;
     end    

     fnrm=fnrmxpp;
     fnrm2=fnrm^2;
     rat=fnrm/fnrm0;
     nridut=nridut+nridu;

%
%   STORING AND PRINTING THE ITERATION'S SUMMARY
% 
     if nargout>=4
        history(itc+1,:)=[itc, fnrm, nridu, gamma, rat];
     end
     if outflag>0
        disp(sprintf('   %3d    %10.5e   %3d     %12.5e   %10.5e   ',itc,fnrm,nridu,full(gamma),rat))
     end
     if (abs(fnrm-fnrm0)<=epsilon*fnrm & fnrm>stoptol)  
        ierr=4;
        break
        return
     end
%
%   UPDATING OF THE TRUST-REGION SIZE
%  
     if rhof>w    
        Delta=max(Delta,delta2*norm(G.*p));
     end
end
%
%   FINAL OUTPUT
% 
sol=x;
if nargin==7 
   jac=feval(Fjac,x);
else
   jac=diffjac(x,F,fx,l,u);
end
grad_old=grad;
grad=jac'*fx;
if scal==1
   if itc<=1 %%%%%%%%%%%%%%% cambiato se itc == 0 altrimenti fallisce
      lambda=max(1.d-2,norm(grad));
   else
      lambda=max(1.d-2,(p'*(grad-grad_old))/(p'*p));
   end
end
[d,dsqr,dmsqr,ind]=dmatrici(x,grad,l,u,scal,lambda);
ndgrad=norm(d.*grad);    
output=[itc,nvf,fnrm,ndgrad,nridut];

if nargout==6
   if ~issparse(jac)
      diagnostic(1)=rank(jac);   
      diagnostic(2:n+1)=svd(jac);
   else
      diagnostic=sprintf('You are working with sparse matrices, \n no diagnostic provided within this framework');
   end
end

if (ierr==0 & fnrm>stoptol)
   if itc==maxit
      ierr=1;
   else
      ierr=2;
   end
end  
if ierr>=1
   disp(sprintf('%s %1.0f','FAILURE, ierr= ',ierr))
   return     
end 

disp(sprintf('\n'))
disp(sprintf('FINAL OUTPUT:'))
disp(sprintf('                 iterations = %d',itc)) 
disp(sprintf('F-evaluations (no Jacobian) = %d',nvf))
disp(sprintf('     final value of ||F||_2 = %10.5e',fnrm))
disp(sprintf('\n'))

%   end of AS_ID function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [jac]=diffjac(x,F,F0,l,u);
%
%   Compute an approximation of the Jacobian matrix of F at x by forward finite differences
%
%   [jac] = diffjac(x,F,F0,l,u);
% 
%   Input:
%      x,F = point and function
%       F0 = F(x), preevaluated
%      l,u = constraints
%
%   Output: 
%      jac = approximated Jacobian matrix
%

n=length(x);
epsnew=sqrt(eps);
for j=1:n
%    
%   CHOICE OF THE STEPLENGHT FOR THE FORWARD DIFFERENCES
% 
    if x(j)==0
       h=epsnew;
    else
       h=epsnew*sign(x(j))*max(abs(x(j)),norm(x,1)/n);
    end
    xhj=x(j)+h; 
%
%
    if (xhj<l(j) |xhj >u(j))
%
%   THE NEW POINT XHJ IS NOT FEASIBLE. IN THIS CASE THE BACKWARD 
%   DIFFERENCE IS USED
%
       h=-h;
       xhj=x(j)+h;
       if (xhj<l(j) |xhj >u(j))
           disp('Function diffjac: loosing of feasibility using both backward and forward differences')
           stop
       end
    end
    xh=x;  xh(j)=xhj;
    F1=feval(F,xh);
    jac(:,j)=(F1-F0)/h;
end

%   end of DIFFJAC function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [d,dsqr,dmsqr,ind]=dmatrici(x,grad,l,u,scal,lambda)

%
%  
%   Compute the scaling matrix D and the matrices inv(D), (D)^(-2)
%
%   [d,dsqr,dmsqr,ind]=dmatrici(x,grad,l,u,scal,lambda);
%
%   Input: 
%    x      = point
%    grad   = gradient of f, preevaluated
%    l,u    = constraints
%    scal   = parameter for choosing the scaling matrix
%    lambda = parameter needed for the choice scal=1;
% 

%   Output: 
%    d     =  array such that d=diag(D(x)).
%    dsqr  =  array such that dsqr=diag((D(x))^1/2).
%    dmsqr =  array such that dmsqr=diag((D(x))^-1/2).
%    ind   =  0 upon successful termination.
%          =  6 an overflow would be generated when computing 
%             the scaling matrix D.
%
ind=0;
n=length(x);
d=ones(n,1); dsqr=ones(n,1);  dmsqr=ones(n,1); 
if scal==1
%
%   HAGER ET AL. SCALING MATRIX
%
   d=(1/lambda)*ones(n,1); 
   for i=1:n
       if grad(i)<0 
          if u(i)~=Inf
             diff=u(i)-x(i);
             d(i)=1/(lambda-grad(i)/diff);
             if d(i)<=1/realmax
                ind=6;
                return
             end
          end
       else
          if l(i)~=-Inf
             diff=x(i)-l(i);
             d(i)=1/(lambda+grad(i)/diff);
             if d(i)<=1/realmax
                ind=6;
                return
             end
          end 
       end
   end
   dsqr=sqrt(d);
   dmsqr=1./dsqr;
elseif scal==0
%
%   COLEMAN-LI SCALING MATRIX
%
       for i=1:n
           if grad(i)<0 
              if u(i)~=Inf
                 diff=u(i)-x(i);
                 sqdiff=sqrt(diff);
                 if diff>=(1/realmax)
                    dmsqr(i)=1/sqdiff;
                    dsqr(i)=sqdiff;
                    d(i)=diff;
                 else
                    ind=6;
                    return
                 end 
              end
           else
              if l(i)~=-Inf
                 diff=x(i)-l(i);
                 sqdiff=sqrt(diff);
                 if diff>=(1/realmax)
                    dmsqr(i)=1/sqdiff;
                    dsqr(i)=sqdiff;
                    d(i)=diff;
                 else
                    ind=6;
                    return
                 end 
              end
           end
       end
elseif scal<0
%
%   KANZOW-KLUG SCALING MATRIX
%
         gamma = -scal;
         chk = (l==-Inf)&(u==Inf);
         a = x-l+gamma*max(0,-grad);
         b = u-x+gamma*max(0,grad);
         c = min(a,b);
         for i=1:n
             if chk(i)==1
                dmsqr(i) = 1;
             else
                if c(i)>1/realmax
                   dmsqr(i) =1/sqrt(c(i));
                else
                   ind=6;
                   return
                end
             end
         end
         dsqr = 1./dmsqr;
         d = dsqr.^2;
elseif scal==2
%
%   USER'S OWN SCALING MATRIX
%
         d = usersd(x,grad,l,u);
         dsqr = sqrt(d);
         dmsqr = 1./dsqr;
end

%   end of DMATRICI function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [a] = isnew
v=version;
cp=0; i1=0; i2=0;
for i=1:length(v)
   if v(i)=='.'
      cp=cp+1;
      if cp==2
         break
      end
   end
   if cp<1
      i1 = i1+1;
      x1(i1)=v(i);
   else
      if v(i)~='.'
         i2 = i2+1;
         x2(i2)=v(i);
      end
   end
end
c1=str2num(x1);
c2=str2num(x2);
a = (c1>=7|(c1==6 & c2>=5));