%*****************************************************************************80
%
%% Evaluates the objective function for problem 15.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = ExtendedPowell(what,x)
  global Nf_eval Ng_eval Nh_eval ExtendedPowell_n ;

  if isempty(ExtendedPowell_n)
    ExtendedPowell_n = 4;
  end

  switch what

  case 'setup'
    if x < 4
      error(['ExtendedPowell(' what ') argument bust be >= 4']) ;
    end
    ExtendedPowell_n = x ;

  case {'name','title'}
    RES = 'The Extended Powell Singular Quartic Function.' ;

  case {'n','size'}
    RES = ExtendedPowell_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    n = ExtendedPowell_n ;
    RES = zeros ( n, 1 );
    for i = 1 : n
      if ( mod ( i, 4 ) == 1 )
        RES(i) = 3.0;
      elseif ( mod ( i, 4 ) == 2 )
        RES(i) = - 1.0;
      elseif ( mod ( i, 4 ) == 3 )
        RES(i) = 0.0;
      else
        RES(i) = 1.0;
      end
    end

  case 'solution'
    RES = zeros(ExtendedPowell_n,1)  ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Extended Powell Singular Quartic Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global ExtendedPowell_n ;
  n = ExtendedPowell_n ;
  f = 0.0;

  for j = 1 : 4 : n

    if ( j + 1 <= n )
      xjp1 = x(j+1);
    else
      xjp1 = 0.0;
    end

    if ( j + 2 <= n )
      xjp2 = x(j+2);
    else
      xjp2 = 0.0;
    end

    if ( j + 3 <= n )
      xjp3 = x(j+3);
    else
      xjp3 = 0.0;
    end

    f1 = x(j) + 10.0 * xjp1;

    if ( j + 1 <= n )
      f2 = xjp2 - xjp3;
    else
      f2 = 0.0;
    end

    if ( j + 2 <= n )
      f3 = xjp1 - 2.0 * xjp2;
    else
      f3 = 0.0;
    end

    if ( j + 3 <= n )
      f4 = x(j) - xjp3;
    else
      f4 = 0.0;
    end

    f = f +        f1 * f1 ...
          +  5.0 * f2 * f2 ...
          +        f3 * f3 * f3 * f3 ...
          + 10.0 * f4 * f4 * f4 * f4;

  end
end

%*****************************************************************************80
function g = eval_g ( x )
  global ExtendedPowell_n ;
  n = ExtendedPowell_n ;
  g = zeros ( 1, n );

  for j = 1 : 4 : n

    if ( j + 1 <= n )
      xjp1 = x(j+1);
    else
      xjp1 = 0.0;
    end

    if ( j + 2 <= n )
      xjp2 = x(j+2);
    else
      xjp2 = 0.0;
    end

    if ( j + 3 <= n )
      xjp3 = x(j+3);
    else
      xjp3 = 0.0;
    end

    f1 = x(j) + 10.0 * xjp1;
    df1dxj = 1.0;
    df1dxjp1 = 10.0;

    if ( j + 1 <= n )
      f2 = xjp2 - xjp3;
      df2dxjp2 = 1.0;
      df2dxjp3 = -1.0;
    else
      f2 = 0.0;
      df2dxjp2 = 0.0;
      df2dxjp3 = 0.0;
    end

    if ( j + 2 <= n )
      f3 = xjp1 - 2.0 * xjp2;
      df3dxjp1 = 1.0;
      df3dxjp2 = -2.0;
    else
      f3 = 0.0;
      df3dxjp1 = 0.0;
      df3dxjp2 = 0.0;
    end

    if ( j + 3 <= n )
      f4 = x(j) - xjp3;
      df4dxj = 1.0;
      df4dxjp3 = -1.0;
    else
      f4 = 0.0;
      df4dxj = 0.0;
      df4dxjp3 = 0.0;
    end

    g(j) = 2.0 * f1 * df1dxj + 10.0 * 4.0 * f4^3 * df4dxj;

    if ( j+1 <= n )
      g(j+1) = 2.0 * f1 * df1dxjp1 + 4.0 * f3^3 * df3dxjp1;
    end

    if ( j+2 <= n )
      g(j+2) = 2.0 * 5.0 * f2 * df2dxjp2 + 4.0 * f3^3 * df3dxjp2;
    end

    if ( j+3 <= n )
      g(j+3) = 2.0 * 5.0 * f2 * df2dxjp3 + 10.0 * 4.0 * f4^3 * df4dxjp3;
    end

  end
end

%*****************************************************************************80
function h = eval_h ( x )
  global ExtendedPowell_n ;
  n = ExtendedPowell_n ;
  h = zeros ( n, n );

  for j = 1 : 4 : n

    if ( j + 1 <= n )
      xjp1 = x(j+1);
    else
      xjp1 = 0.0;
    end

    if ( j + 2 <= n )
      xjp2 = x(j+2);
    else
      xjp2 = 0.0;
    end

    if ( j + 3 <= n )
      xjp3 = x(j+3);
    else
      xjp3 = 0.0;
    end

    f1 = x(j) + 10.0 * xjp1;
    df1dxj = 1.0;
    df1dxjp1 = 10.0;

    if ( j + 1 <= n )
      f2 = xjp2 - xjp3;
      df2dxjp2 = 1.0;
      df2dxjp3 = -1.0;
    else
      f2 = 0.0;
      df2dxjp2 = 0.0;
      df2dxjp3 = 0.0;
    end

    if ( j + 2 <= n )
      f3 = xjp1 - 2.0 * xjp2;
      df3dxjp1 = 1.0;
      df3dxjp2 = -2.0;
    else
      f3 = 0.0;
      df3dxjp1 = 0.0;
      df3dxjp2 = 0.0;
    end

    if ( j + 3 <= n )
      f4 = x(j) - xjp3;
      df4dxj = 1.0;
      df4dxjp3 = -1.0;
    else
      f4 = 0.0;
      df4dxj = 0.0;
      df4dxjp3 = 0.0;
    end

    h(j  ,j  ) = 2.0 * df1dxj * df1dxj + 120.0 * f4 * f4 * df4dxj * df4dxj;

    if ( j + 1 <= n )
      h(j  ,j+1) = 2.0 * df1dxj * df1dxjp1;
      h(j+1,j  ) = 2.0 * df1dxj * df1dxjp1;
      h(j+1,j+1) = 2.0 * df1dxjp1 * df1dxjp1 + 12.0 * f3 * f3 * df3dxjp1 * df3dxjp1;
    end

    if ( j + 2 <= n )
      h(j  ,j+2) = 0.0;
      h(j+2,j  ) = 0.0;
      h(j+1,j+2) = 12.0 * f3 * f3 * df3dxjp2 * df3dxjp1;
      h(j+2,j+1) = 12.0 * f3 * f3 * df3dxjp1 * df3dxjp2;
      h(j+2,j+2) = 10.0 * df2dxjp2 * df2dxjp2 + 12.0 * f3 * f3 * df3dxjp2 * df3dxjp2;
    end

    if ( j + 3 <= n )
      h(j  ,j+3) = 120.0 * f4 * f4 * df4dxj * df4dxjp3;
      h(j+3,j  ) = 120.0 * f4 * f4 * df4dxj * df4dxjp3;
      h(j+1,j+3) = 0.0;
      h(j+3,j+1) = 0.0;
      h(j+2,j+3) = 10.0 * df2dxjp3 * df2dxjp2;
      h(j+3,j+2) = 10.0 * df2dxjp2 * df2dxjp3;
      h(j+3,j+3) = 10.0 * df2dxjp3 * df2dxjp3 + 120.0 * f4 * f4 * df4dxjp3 * df4dxjp3;
    end

  end
end
