%*****************************************************************************80
%
%% Evaluates the objective function for problem 29.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Zbigniew Michalewicz,
%    Genetic Algorithms + Data Structures = Evolution Programs,
%    Third Edition,
%    Springer Verlag, 1996,
%    ISBN: 3-540-60676-9,
%    LC: QA76.618.M53.
%
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = GoldsteinPrice(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Goldstein Price Polynomial.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -0.5, +0.25 ]' ;

  case 'solution'
    RES = [ 0.0, -1.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Goldstein Price Polynomial, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  a = x(1) + x(2) + 1.0;
  b = 19.0 - 14.0 * x(1) + 3.0 * x(1) * x(1) - 14.0 * x(2) + 6.0 * x(1) * x(2) + 3.0 * x(2) * x(2);
  c = 2.0 * x(1) - 3.0 * x(2);
  d = 18.0 - 32.0 * x(1) + 12.0 * x(1) * x(1) + 48.0 * x(2) - 36.0 * x(1) * x(2) + 27.0 * x(2) * x(2);
  f = ( 1.0 + a * a * b ) * ( 30.0 + c * c * d );
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  a = x(1) + x(2) + 1.0;

  b = 19.0 - 14.0 * x(1) + 3.0 * x(1)^2 - 14.0 * x(2) + 6.0 * x(1) * x(2) + 3.0 * x(2)^2;

  dbdx1 = - 14.0 + 6.0 * x(1) + 6.0 * x(2);
  dbdx2 = - 14.0 + 6.0 * x(1) + 6.0 * x(2);

  c = 2.0 * x(1) - 3.0 * x(2);

  d = 18.0 - 32.0 * x(1) + 12.0 * x(1)^2 + 48.0 * x(2) - 36.0 * x(1) * x(2) + 27.0 * x(2)^2;
  dddx1 = - 32.0 + 24.0 * x(1) - 36.0 * x(2);
  dddx2 = 48.0 - 36.0 * x(1) + 54.0 * x(2);

  g(1) = ( 1.0 + a^2 * b ) * ( 4.0 * c * d + c^2 * dddx1 ) + ( 2.0 * a * b + a^2 * dbdx1 ) * ( 30.0 + c^2 * d );
  g(2) = ( 1.0 + a^2 * b ) * ( -6.0 * c * d + c^2 * dddx2 ) + ( 2.0 * a * b + a^2 * dbdx2 ) * ( 30.0 + c^2 * d );
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 ) ;

  a = x(1) + x(2) + 1.0;

  b = 19.0 - 14.0 * x(1) + 3.0 * x(1)^2 - 14.0 * x(2) + 6.0 * x(1) * x(2) + 3.0 * x(2)^2;
  e = - 14.0 + 6.0 * x(1) + 6.0 * x(2) ;
  c = 2.0 * x(1) - 3.0 * x(2) ;
  d = 18.0 - 32.0 * x(1) + 12.0 * x(1)^2 + 48.0 * x(2) - 36.0 * x(1) * x(2) + 27.0 * x(2)^2;
  r = - 32.0 + 24.0 * x(1) - 36.0 * x(2);
  s = 48.0 - 36.0 * x(1) + 54.0 * x(2);

  h(1,1) = 2.0 * ( 2.0 * a * b + a^2 * e ) ...
    * ( 4.0 * c * d + c^2 * r ) + ( 1.0 + a^2 * b ) ...
    * ( 8.0 * d + 4.0 * c * r + 4.0 * c * r + 24.0 * c^2 ) ...
    + ( 2.0 * b + 2.0 * a * e + 2.0 * a * e + 6.0 * a^2 ) ...
    * ( 30.0 + c^2 * d );

  h(1,2) = ( 2.0 * a * b + a^2 * e ) ...
    * ( -2.0 * c * d + c^2 * ( r + s ) ) ...
    + ( 1.0 + a^2 * b ) ...
    * ( -12.0 * d + 4.0 * c * s -6.0 * c * r - 36.0 * c^2 ) ...
    + ( 2.0 * b + 4.0 * a * e + 6.0 * a^2 ) ...
    * ( 30.0 + c^2 * d );

  h(2,1) = ( 2.0 * a * b + a^2 * e ) ...
    * ( -2.0 * c * d + c^2 * ( s + r ) ) ...
    + ( 1.0 + a^2 * b ) ...
    * ( -12.0 * d - 6.0 * c * r + 4.0 * c * s - 36.0 * c^2 ) ...
    + ( 2.0 * b + 4.0 * a * e + 6.0 * a^2 ) ...
    * ( 30.0 + c^2 * d );

  h(2,2) = 2.0 * ( 2.0 * a * b + a^2 * e ) ...
    * ( -6.0 * c * d + c^2 * s ) ...
    + ( 1.0 + a^2 * b ) ...
    * ( 18.0 * d - 6.0 * c * s - 6.0 * c * s + 54.0 * c^2 ) ...
    + ( 2.0 * b + 2.0 * a * e + 2.0 * a * e + 6.0 * a^2 ) ...
    * ( 30.0 + c^2 * d );

end
