%*****************************************************************************80
%
%% HelicalValley evaluates the objective function for problem 1.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = HelicalValley(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Fletcher and Powell helical valley function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -1.0, 0.0, 0.0 ]' ;

  case 'solution'
    RES = [ 1.0, 0.0, 0.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['Fletcher and Powell helical valley function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function th = eval_th ( x )
  if ( 0.0 < x(1) )
    th = 0.5 * atan2 ( x(2), x(1) ) / pi ;
  elseif ( x(1) < 0.0 )
    th = 0.5 * atan2 ( x(2), x(1) ) / pi + 0.5;
  elseif ( 0.0 < x(2) )
    th = 0.25 ;
  elseif ( x(2) < 0.0 )
    th = -0.25 ;
  else
    th = 0.0 ;
  end
end

%*****************************************************************************80
function f = eval_f ( x )
  th = eval_th ( x );
  f = 100.0 * ( x(3) - 10.0 * th )^2 ...
    + 100.0 * ( sqrt ( x(1) * x(1) + x(2) * x(2) ) - 1.0 )^2 ...
    + x(3) * x(3);
end

%*****************************************************************************80
function g = eval_g ( x )
  th = eval_th ( x ) ;

  r  = sqrt ( x(1) * x(1) + x(2) * x(2) );
  t  = x(3) - 10.0 * th;
  s1 = 5.0 * t / ( pi * r * r );

  g = [ 200.0 * ( x(1) - x(1) / r + x(2) * s1 ), ...
        200.0 * ( x(2) - x(2) / r - x(1) * s1 ), ...
        2.0 * ( 100.0 * t + x(3) ) ] ;
end

%*****************************************************************************80
function h = eval_h ( x )

  th = eval_th ( x ) ;
  h  = zeros ( 3, 3 );

  h(1,1) = 200.0 - 200.0 * x(2)^2 ...
         * ( 1.0 / sqrt ( x(1)^2 + x(2)^2 )^3 ...
         - 25.0 / ( pi * ( x(1)^2 + x(2)^2 ) )^2 ) ...
         - 2000.0 * x(1) * x(2) * ( x(3) - 10.0 * th ) ...
         / ( pi * ( x(1)^2 + x(2)^2 )^2 );

  h(1,2) = 200.0 * x(1) * x(2) / sqrt ( x(1)^2 + x(2)^2 )^3 ...
         + 1000.0 / ( pi * ( x(1)^2 + x(2)^2 )^2 ) ...
         * ( ( x(3) - 10.0 * th ) * ( x(1)^2 - x(2)^2 ) ...
         - 5.0 * x(1) * x(2) / pi );

  h(1,3) = 1000.0 * x(2) / ( pi * ( x(1)^2 + x(2)^2 ) );

  h(2,1) = 200.0 * x(1) * x(2) / sqrt ( x(1)^2 + x(2)^2 )^3 ...
         + 1000.0 / ( pi * ( x(1)^2 + x(2)^2 )^2 ) ...
         * ( ( x(3) - 10.0 * th ) * ( x(1)^2 - x(2)^2 ) ...
         - 5.0 * x(1) * x(2) / pi );

  h(2,2) = 200.0 - 200.0 * x(1)^2 ...
         * ( 1.0 / sqrt ( x(1)^2 + x(2)^2 )^3 ...
         - 25.0 / ( pi * ( x(1)^2 + x(2)^2 ) )^2 ) ...
         + 2000.0 * x(1) * x(2) * ( x(3) - 10.0 * th ) ...
         / ( pi * ( x(1)^2 + x(2)^2 )^2 );

  h(2,3) = - 1000.0 * x(1) / ( pi * ( x(1)^2 + x(2)^2 ) );

  h(3,1) = 1000.0 * x(2) / ( pi * ( x(1)^2 + x(2)^2 ) );
  h(3,2) = - 1000.0 * x(1) / ( pi * ( x(1)^2 + x(2)^2 ) );
  h(3,3) = 202.0;
end
