%
% Trust region minimization with exact solution of trusted step.
%
%  USAGE:
%
%  x = TrustRegion( fun, x0, epsi )
%
%  or
%
% [x,g,radius] = TrustRegion( fun, x0, epsi )
%
function [ierr,X,F,NG] = TrustRegion( fun, x0, varargin )
  ierr = 0 ;
  opts = { 'tol', 'maxiter', 'maxsubiter', 'beta1', 'beta2' };
  defs = {  1E-8,       100,           20,     0.1,    0.99 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  beta1      = OPTS.beta1 ;
  beta2      = OPTS.beta2 ;
  maxiter    = OPTS.maxiter ;
  maxsubiter = OPTS.maxsubiter ;
  tol        = OPTS.tol ;

  x = x0 ;
  f = feval( fun, 'f',   x ) ;
  g = feval( fun, 'Df',  x ) ;
  H = feval( fun, 'DDf', x ) ;

  radius = norm(g) ;

  X  = [] ;
  F  = [] ;
  NG = [] ;

  for k=1:maxiter

    % compute model
    g = feval( fun, 'Df',  x ) ;
    H = feval( fun, 'DDf', x ) ;

    % save values
    X  = [ X x ] ;
    F  = [ F f ] ;
    NG = [ NG norm(g,'inf') ] ;

    done    = false ;
    subiter = 0 ;
    while ~done
      % compute the step
      s = DogLegStep( radius, g', H ) ;
      if ~isfinite(s)
        ierr = 3 ;
        return ;
      end
      
      xnew = x + s ;
      fnew = feval( fun, 'f', xnew ) ;

      if norm(s,inf) < tol & norm(g,inf) < tol
        disp('solution found!');
        return ;
      end;
    
      % check the reduction
      ared = f - fnew ; 
      pred = - dot( g, s )  - 0.5 * dot(s,H*s) ;
      rho  = ared / pred ;
      
      if rho >= beta2 % very successfull augment radius
        done   = true ;
        x      = xnew ;
        f      = fnew ;
        radius = max(radius, 2*norm(s)) ;
      elseif rho >= beta1 % successful 
        done   = true ;
        x      = xnew ;
        f      = fnew ;
      else
        fprintf(1, 'reject iter=%d radius=%g',k,radius) ;
        radius = radius / 2 ;
      end
      subiter = subiter + 1 ;
      if ~done & subiter >= maxsubiter ;
        ierr = 2 ;
        return ;
      end
    end
  end ;
  ierr = 1 ;
end

%
% Trust region minimization with DogLeg trusted step.
%
%  USAGE:
%
%  x = DogLeg( fun, Grad, Hess, x0, epsi )
%
%  or
%
% [x,g,radius] = DogLeg( fun, Grad, Hess, x0, epsi )
%
function s = DogLegStep( radius, g, H )

  Sg = -g*(dot(g,g)/dot(g,H*g)) ;
  a  = dot(Sg,Sg) ;
  if a >= radius^2 
    s = (radius/sqrt(a))*Sg ;
    return ;
  end
  
  if rcond(H) < 1E-10
    s = Sg ;
    return ;    
  end

  Sn = -H\g ;
  b  = dot(Sn,Sn) ;

  if b > radius^2
    c     = dot( Sg - Sn, Sg - Sn );
    d     = (a+b-c) / 2 ;
    alpha = (b-radius^2)/(b-d+sqrt(d^2-a*b+c*radius^2)) ;
    s     = alpha * Sg + (1-alpha) * Sn ;
  else
    s = Sn ;    
  end
end
