%
% Trust region minimization with exact solution of trusted step.
%
%  USAGE:
%
%  x = LevenbergMarquardt( fun, Grad, Hess, x0, epsi )
%
%  or
%
% [x,g,radius] = LevenbergMarquardt( fun, Grad, Hess, x0, epsi )
%
function [ierr,X,FF,NG] = LevenbergMarquardtMinimization( fun, x0, varargin )  

  ierr = 0 ;
  opts = { 'tol', 'maxiter', 'maxsubiter', 'beta1', 'beta2' } ;
  defs = {  1E-8,       100,           50,    0.25,    0.75 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  beta1      = OPTS.beta1 ;
  beta2      = OPTS.beta2 ;
  maxiter    = OPTS.maxiter ;
  maxsubiter = OPTS.maxsubiter ;
  tol        = OPTS.tol ;

  x  = x0 ;

  Id = eye(length(x));

  X  = [] ;
  FF = [] ;
  NG = [] ;

  % compute model
  f  = feval( fun, 'f',   x ) ;
  g  = feval( fun, 'Df',  x ) ;
  H  = feval( fun, 'DDf', x ) ;
  mu = norm( H, 'fro' ) ;

  for k=1:maxiter

    first = true ;
    % save values
    X  = [ X  x ] ;
    FF = [ FF f ] ;
    NG = [ NG norm(g,'inf') ] ;

    for kk=1:maxsubiter

      [Q,R] = qr(H+mu*Id) ;

      if not(all(isfinite(Q))) | not(all(isfinite(R)))
        fprintf(1,'\n\nfound not regular number in H+mu*Id\n\n') ;
        ierr = 1 ;
        return ;
      end

      s = - R\((g*Q)') ;

      % check if it is a descent direction
      alpha = dot(s,g) ;
      if alpha >= 0
        % if not a descent direction reduce trust region
        mu = 2 * mu ;
        fprintf(1,'reject (not descent) iter=%d mu=%g\n',k,mu) ;
        continue ;
      end

      % perform step
      xnew = x + s ;
      fnew = feval( fun, 'f',  xnew ) ;
      gnew = feval( fun, 'Df', xnew ) ;

      if norm(s,inf) < tol && norm(gnew,inf) < tol
        X  = [ X  xnew ] ;
        FF = [ FF fnew ] ;
        NG = [ NG norm(gnew,'inf') ] ;
        fprintf(1, '\n\nsolution found at iter %d |g|=%g!\n\n',k,norm(gnew,inf));
        return ;
      end;

      if mu > 1E100
        disp('lambda too big!');
        ierr = 2 ;
        return ;
      end;

      % check the reduction
      ared = f - fnew ;
      pred = - alpha - 0.5 * (dot(s,H*s+mu*s)) ;
      rho  = ared / pred ;

      if rho < 0
        % no reduction, reduce trust region
        mu = 2 * mu ;
        fprintf(1,'reject (no reduction) iter=%d mu=%g rho=%g\n',k,mu,rho) ;
        continue ;
      else
        fprintf(1,'accept iter=%d mu=%g rho=%g\n',k,mu,rho) ;
      end
      
      % accept step
      x = xnew ;
      f = fnew ;
      g = gnew ;
      H = feval( fun, 'DDf', x ) ;

      % enlarge or reduce trust region depending on ratio rho.
      if rho < beta1
        % too few reduction, reduce trust region
        mu = 2 * mu ;
      elseif rho > beta2
        % very good reduction, enlarge trust region
        mu = mu/2 ;
      end
      break ;
    end % end while
  end ;
  ierr = 3 ;
end
